include Counter.inc

;**********************CODE SEGMENT*********************************
.code 
start:;START PROG HERE. 
	invoke GetModuleHandle, NULL
	mov    hInstance,eax

	;Convert this thread to a fiber.
	invoke ConvertThreadToFiber,NULL
	mov g_FiberInfo.pFiberUI,eax
	
	;Create the application's UI window.	
	invoke InitCommonControls
	invoke CreateDialogParam,hInstance,IDD_COUNTER,NULL,addr DlgProc,NULL
	mov g_FiberInfo.hwnd,eax
	 
	 ;Update the window showing which fiber is executing.
	 invoke SetDlgItemText,g_FiberInfo.hwnd, IDC_FIBER,SADD("User interface")
	
	;Initially, there is no background processing to be done.
	MOV g_FiberInfo.bps,BPS_DONE
	
	;While the UI window still exists...
	 
	.while !fQuit
		;UI messages are higher priority than background processing.
		invoke PeekMessage,addr msg,NULL,0,0,PM_REMOVE
	
		.if eax		
			; If a message exists in the queue, process it.		
			.if msg.message == WM_QUIT
				mov fQuit,TRUE
			.endif		
			invoke IsDialogMessage,g_FiberInfo.hwnd,addr msg
			.if eax==0
				invoke TranslateMessage,addr msg
				invoke DispatchMessage,addr msg
			.endif
		.else
			;No UI msgs exist; check the state of the background 
			;processing.			
			mov eax,g_FiberInfo.bps
			.if eax == BPS_DONE
				;No background processing to do; wait for a UI event.
				invoke WaitMessage
		
			.elseif eax == BPS_STARTOVER
				;User changed the count; restart the background 
				;processing.
				
				.if pFiberCounter != NULL
					;A recalculation fiber exists; delete it so that
                    ; background processing starts over from the 
					;beginning.
					invoke DeleteFiber,pFiberCounter
					mov pFiberCounter ,NULL
				.endif
			
				;Create a new recalc fiber that starts from the 
				;beginning.
				invoke CreateFiber,0,addr FiberFunc,addr g_FiberInfo			
				mov pFiberCounter,eax 
				
				; The background processing started; it should continue.
				mov g_FiberInfo.bps,BPS_CONTINUE
			
				;Fall through to BPS_CONTINUE case...
			
			.elseif eax == BPS_CONTINUE
				;Allow the background processing to execute...
				invoke SwitchToFiber, pFiberCounter
						
				;The background processing has been paused 
				;(because a UI message showed up) or has been 
				;stopped (because the counting has completed).
				
				;Update the window showing which fiber is executing.
				invoke  SetDlgItemText,g_FiberInfo.hwnd, IDC_FIBER, SADD("User interface")
										
				.IF g_FiberInfo.bps == BPS_DONE
					;the background processing ran to completion. Delete 
					;the
                    ;fiber so that processing will restart next time.	
                    INVOKE DeleteFiber,pFiberCounter
                    mov pFiberCounter ,NULL
     			.endif
     		.endif;switch on background processing state
     	
     	.endif;No UI messages exist
     .endw;while the window still exists
     
     invoke DestroyWindow,g_FiberInfo.hwnd    						
	 invoke ExitProcess,NULL

;*************
FiberFunc  proc uses esi pFiberInfo:dword
local nCount:dword		
	push pFiberInfo
	pop esi
	assume esi:ptr FIBERINFO

	;Update the window showing which fiber is executing.
	invoke SetDlgItemText,[esi].hwnd, IDC_FIBER, SADD("Recalculation")
	
	; Get the current count in the EDIT control.
	invoke GetDlgItemInt,[esi].hwnd, IDC_COUNT, NULL, FALSE	
	mov nCount,eax

	;Count from 0 to nCount, updating the STATIC control.
	xor ecx,ecx
	.while ecx<= nCount
		push ecx
		;UI events have higher priority than counting.
        ; If there are any UI events, handle them ASAP.
		invoke GetQueueStatus,QS_ALLEVENTS
		shr eax,16
		.if eax!=0
			
			;The UI fiber has something to do; temporarily
            ; pause counting and handle the UI events.			
			invoke SwitchToFiber,[esi].pFiberUI
						
			;The UI has no more events; continue counting.
			invoke SetDlgItemText,[esi].hwnd, IDC_FIBER, SADD("Recalculation")
		.endif
		
		;Update the STATIC control with the most recent count.
		pop ecx
		push ecx
		invoke SetDlgItemInt,[esi].hwnd, IDC_ANSWER, ecx, FALSE

		;Sleep for a while to exaggerate the effect; remove 
        ; the call to Sleep in production code.
		invoke Sleep,200
		
		pop ecx
		inc ecx
	.endw
	
	;Indicate that counting is complete.
	mov [esi].bps,BPS_DONE

	;Reschedule the UI thread. When the UI thread is running
    ; and has no events to process, the thread is put to sleep.
    ; NOTE: If we just allow the fiber function to return,
    ; the thread and the UI fiber die -- we don't want this!
	invoke SwitchToFiber,[esi].pFiberUI
    ret
FiberFunc endp

;***********DIALOG MESSAGE PROCEDURE*****************************
DlgProc	proc hWnd:DWORD,uMsg:DWORD,wParam:DWORD,lParam:DWORD

		mov	eax,uMsg
		.if	eax == WM_CLOSE
			invoke	PostQuitMessage,NULL
		.elseif	eax == WM_INITDIALOG
			invoke LoadIcon,hInstance,IDI_COUNTER
			push eax
			invoke SendMessage,hWnd,WM_SETICON,ICON_SMALL,eax		
			pop eax
			invoke SendMessage,hWnd,WM_SETICON,ICON_BIG,eax
			
			invoke SetDlgItemInt,hWnd, IDC_COUNT, 0, FALSE
			
		.elseif	eax == WM_COMMAND
			mov	eax,wParam
			and eax,0ffffh
			.if	eax == IDCANCEL
				invoke	PostQuitMessage,NULL
				
			.elseif eax == IDC_COUNT
				mov eax,wParam
				shr eax,16
				.if eax == EN_CHANGE					
					
					;When the user changes the count, start the 
                    ;background processing over from the beginning.
                    mov g_FiberInfo.bps,BPS_STARTOVER
				.endif				
			.endif
		.else
			mov	eax,FALSE
			ret
		.endif		   
		mov	eax,TRUE
		ret
		
DlgProc	endp
;********************************************************************
end start